using System.Formats.Tar;
using System.Net;
using System.Runtime.InteropServices;
using System.Security.AccessControl;
using CmlLib.Core;
using CmlLib.Core.Auth;
using CmlLib.Core.ProcessBuilder;

namespace PlombirLauncher
{
    /*
        Basically a class to manage everything launcher related.
        TODO - minecraft downloading from webserver
    */
    public class Launcher
    {
        private readonly string _version;
        private readonly string _nickname;
        static private MinecraftPath _mcPath;
        static private MinecraftLauncher _mcLauncher;

        public int DownloadProgress;
        public string DownloadStatus;

        public Launcher(string userName, string version, string? location = null)
        {
            _version = version;
            _nickname = userName;
            _mcPath = new($"{location}/{version}/minecraft");
            _mcLauncher = new(_mcPath);
        }

        // Using it for forge loading because i don't know event-based async.
        //static TaskCompletionSource<bool> tcs = new();


        // Not used. Because launcher supports only VANILLA.
        // public async Task FetchMinecraft()
        // {

        //     string archieve = "mc-archieve.tar.xz";
        //     string runtimePath = "runtime";

        //     System.Uri download_uri = new("https://dixxe.top/static/distro/cmllibcore-minecraft-forge-1201.tar.xz");
        //     WebClient client = new();

        //     if ((!File.Exists(archieve)) && (!Directory.Exists(runtimePath)))
        //     {
        //         DownloadStatus = "Fetching deploy-minecraft from host...";
        //         Console.WriteLine(DownloadStatus);

        //         // Implement async download
        //         client.DownloadFileAsync(download_uri, archieve);
        //         client.DownloadDataCompleted += (o, e) =>
        //         {
        //             DownloadStatus = "Download completed";
        //             Console.WriteLine(DownloadStatus);
        //         };
        //         client.DownloadProgressChanged += (o, e) =>
        //         {
        //             // Curentlly this scope doing nothing
        //             Console.WriteLine($"{e.ProgressPercentage}% downloaded");
        //             DownloadProgress = e.ProgressPercentage;
        //             DownloadStatus = "Fetching modpack...";

        //             if (DownloadProgress == 100)
        //             {
        //                 tcs.SetResult(true);
        //             }

        //         };
        //         // Waiting for archieve to download.
        //         await tcs.Task;
        //     }

        //     DownloadStatus = "Unarchiving modpack...";
        //     unarchiveMinecraft(archieve, runtimePath);

        // }

        // Not used. Because launcher supports only VANILLA.
        // private void unarchiveMinecraft(string archieveName, string path)
        // {
        //     if (Directory.Exists(path)) { return; }
        //     Console.WriteLine(DownloadStatus);
        //     Directory.CreateDirectory(path);
        //     Utils.UnzipTarGzFile(archieveName, path);

        //     if (RuntimeInformation.IsOSPlatform(OSPlatform.Linux))
        //     {
        //         Utils.LinuxExec($"chmod -R +x {path}/");
        //         Console.WriteLine("Execute permissions added successfully.");
        //     }

        //     File.Delete(archieveName);
        // }

        async public Task BuildLauncher(CancellationToken token)
        {
            _mcLauncher.ByteProgressChanged += (_, args) =>
            {
                Console.WriteLine($"{(int)(args.ProgressedBytes * 0.000001)} MBytes / {(int)(args.TotalBytes * 0.000001)} MBytes");
                DownloadProgress = (int)(args.ProgressedBytes * 100 / args.TotalBytes);
            };

            // Before installAsync I should fetch lightweight minecraft instance from webserver
            // Then installAsync will download any runtime needed stuff.
            // This method is FetchMinecraft()
            // Currently this is only one viable option for forge installations.
            // But it requires server, so forge (and all mod loaders) is disabled.
            //await FetchMinecraft();

            DownloadStatus = "Installing minecraft...";
            Console.WriteLine(DownloadStatus);
            await _mcLauncher.InstallAsync(_version);
            DownloadStatus = "Finished!";
            Console.WriteLine(DownloadStatus);
            return;
        }

        async public Task RunLauncher()
        {
            var process = await _mcLauncher.BuildProcessAsync(_version, new MLaunchOption
            {

                Session = MSession.CreateOfflineSession(_nickname),
                MaximumRamMb = 4096,

            }) ?? throw new Exception("Failed to start minecraft process!");

            var processUtil = new ProcessWrapper(process);
            processUtil.OutputReceived += (s, e) => Console.WriteLine(e);
            processUtil.StartWithEvents();

            await processUtil.WaitForExitTaskAsync();
        }

    }
}
